{% extends "base.html" %}
{% load static %}

{% block content %}
  <!-- Modal for server update -->
  <div class="modal fade" id="in-progress">
    <div class="modal-dialog">
      <div class="modal-content">
        <div class="modal-header">
          <h4 class="modal-title">Server Update In-Progress <img class="pull-right" src="{% static 'img/in-progress.gif' %}" /></h4>
        </div>
        <div class="modal-body">
            <div class="progress">
                <div class="progress-bar" style="width: 0%;"></div>
            </div>
        </div>

      </div><!-- /.modal-content -->
    </div><!-- /.modal-dialog -->
  </div><!-- /.modal -->
{% endblock %}