# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-11 01:29
from __future__ import unicode_literals

from django.db import migrations
import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0012_model_sync'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, unique=True),
        ),

        # Removing placeholder fields. See 0010
        migrations.RemoveField(
            model_name='subscription',
            name="plan_stripe_id",
        ),
        migrations.RemoveField(
            model_name='invoice',
            name='charge_stripe_id'
        ),
        migrations.RemoveField(
            model_name='invoiceitem',
            name="plan_stripe_id",
        ),
        migrations.RemoveField(
            model_name='invoiceitem',
            name='subscription_stripe_id'
        ),
    ]
