# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-06-25 17:51
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0013_sync_cleanup'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charge',
            name='receipt_sent',
            field=models.BooleanField(default=False, help_text='Whether or not a receipt was sent for this charge.'),
        ),
        migrations.AlterField(
            model_name='charge',
            name='source',
            field=models.ForeignKey(help_text='The source used for this charge.', null=True,
                                    on_delete=django.db.models.deletion.CASCADE, related_name='charges',
                                    to='djstripe.StripeSource'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='application_fee_percent',
            field=djstripe.fields.StripePercentField(
                decimal_places=2,
                help_text='A positive decimal that represents the fee percentage of the subscription invoice amount \
                that will be transferred to the application owner\xe2\x80\x99s Stripe account each billing period.',
                max_digits=5, null=True,
                validators=[django.core.validators.MinValueValidator(1.0),
                            django.core.validators.MaxValueValidator(100.0)]),
        ),
    ]
