# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 14:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


DJSTRIPE_SUBSCRIBER_MODEL = getattr(settings, "DJSTRIPE_SUBSCRIBER_MODEL", settings.AUTH_USER_MODEL)


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0021_auto_20170107_0813'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='subscriber',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to=DJSTRIPE_SUBSCRIBER_MODEL, related_name="customer"),
        ),
    ]
